@@ -1,7 +1,9 @@
+from sqlalchemy import exc as sqla_exc
 from sqlalchemy import text
 
 from alembic.testing import exclusions
 from alembic.testing.requirements import SuiteRequirements
+from alembic.util import compat
 from alembic.util import sqla_compat
 
 
@@ -37,6 +39,17 @@ class DefaultRequirements(SuiteRequirements):
             )
         )
 
+    @property
+    def non_native_boolean_check_constraint(self):
+        """backend creates a check constraint for booleans if enabled"""
+
+        return exclusions.only_on(
+            exclusions.LambdaPredicate(
+                lambda config: not config.db.dialect.supports_native_boolean
+                and config.db.dialect.non_native_boolean_check_constraint
+            )
+        )
+
     @property
     def check_constraints_w_enforcement(self):
         return exclusions.fails_on(["mysql", "mariadb"])
@@ -51,12 +64,6 @@ class DefaultRequirements(SuiteRequirements):
         """foreign key constraints always have names in the DB"""
         return exclusions.fails_on("sqlite")
 
-    @property
-    def no_name_normalize(self):
-        return exclusions.skip_if(
-            lambda config: config.db.dialect.requires_name_normalize
-        )
-
     @property
     def reflects_fk_options(self):
         return exclusions.open()
@@ -91,11 +98,23 @@ class DefaultRequirements(SuiteRequirements):
 
     @property
     def fk_ondelete_is_reflected(self):
-        return exclusions.fails_on(["mssql"])
+        def go(config):
+            if exclusions.against(config, "mssql"):
+                return not sqla_compat.sqla_14_26
+            else:
+                return False
+
+        return exclusions.fails_if(go)
 
     @property
     def fk_onupdate_is_reflected(self):
-        return self.fk_onupdate + exclusions.fails_on(["mssql"])
+        def go(config):
+            if exclusions.against(config, "mssql"):
+                return not sqla_compat.sqla_14_26
+            else:
+                return False
+
+        return self.fk_onupdate + exclusions.fails_if(go)
 
     @property
     def fk_onupdate(self):
@@ -105,7 +124,15 @@ class DefaultRequirements(SuiteRequirements):
 
     @property
     def reflects_unique_constraints_unambiguously(self):
-        return exclusions.fails_on(["mysql", "mariadb", "oracle"])
+        return exclusions.fails_on(["mysql", "mariadb", "oracle", "mssql"])
+
+    @property
+    def reports_unique_constraints_as_indexes(self):
+        return exclusions.only_on(["mysql", "mariadb", "oracle"])
+
+    @property
+    def reports_unnamed_constraints(self):
+        return exclusions.skip_if(["sqlite"])
 
     @property
     def reflects_indexes_w_sorting(self):
@@ -150,6 +177,10 @@ class DefaultRequirements(SuiteRequirements):
     def mssql(self):
         return exclusions.only_on(["mssql"])
 
+    @property
+    def covering_indexes(self):
+        return exclusions.only_on(["postgresql >= 11", "mssql"])
+
     @property
     def postgresql_uuid_ossp(self):
         def check_uuid_ossp(config):
@@ -209,12 +240,6 @@ class DefaultRequirements(SuiteRequirements):
             ["postgresql < 12", "sqlite < 3.31", "mysql < 5.7"]
         )
 
-    @property
-    def computed_reflects_normally(self):
-        return exclusions.only_if(
-            exclusions.BooleanPredicate(sqla_compat.has_computed_reflection)
-        )
-
     @property
     def computed_reflects_as_server_default(self):
         # note that this rule will go away when SQLAlchemy correctly
@@ -288,6 +313,54 @@ class DefaultRequirements(SuiteRequirements):
         else:
             return False
 
+    @property
+    def json_type(self):
+        return exclusions.only_on(
+            [
+                lambda config: exclusions.against(config, "mysql")
+                and (
+                    (
+                        not config.db.dialect._is_mariadb
+                        and exclusions.against(config, "mysql >= 5.7")
+                    )
+                    or (
+                        config.db.dialect._mariadb_normalized_version_info
+                        >= (10, 2, 7)
+                    )
+                ),
+                "mariadb>=10.2.7",
+                "postgresql >= 9.3",
+                self._sqlite_json,
+                self._mssql_json,
+            ]
+        )
+
+    def _mssql_json(self, config):
+        if not sqla_compat.sqla_14:
+            return False
+        else:
+            return exclusions.against(config, "mssql")
+
+    def _sqlite_json(self, config):
+        if not sqla_compat.sqla_14:
+            return False
+        elif not exclusions.against(config, "sqlite >= 3.9"):
+            return False
+        else:
+            with config.db.connect() as conn:
+                try:
+                    return (
+                        conn.execute(
+                            text(
+                                """select json_extract('{"foo": "bar"}', """
+                                """'$."foo"')"""
+                            )
+                        ).scalar()
+                        == "bar"
+                    )
+                except sqla_exc.DBAPIError:
+                    return False
+
     @property
     def identity_columns(self):
         # TODO: in theory if these could come from SQLAlchemy dialects
@@ -313,3 +386,27 @@ class DefaultRequirements(SuiteRequirements):
         return exclusions.only_if(
             lambda config: not getattr(config.db, "_is_future", False)
         )
+
+    @property
+    def stubs_test(self):
+        def requirements():
+            try:
+                import black  # noqa
+                import zimports  # noqa
+
+                return False
+            except Exception:
+                return True
+
+        imports = exclusions.skip_if(
+            requirements, "black and zimports are required for this test"
+        )
+        version = exclusions.only_if(
+            lambda _: compat.py311, "python 3.11 is required"
+        )
+
+        sqlalchemy = exclusions.only_if(
+            lambda _: sqla_compat.sqla_2, "sqlalchemy 2 is required"
+        )
+
+        return imports + version + sqlalchemy
