@@ -1,11 +1,10 @@
-# coding: utf-8
-
 from contextlib import contextmanager
 import os
 import re
 import textwrap
 
 import sqlalchemy as sa
+from sqlalchemy import pool
 
 from alembic import command
 from alembic import testing
@@ -28,10 +27,9 @@ from alembic.testing.env import write_script
 from alembic.testing.fixtures import capture_context_buffer
 from alembic.testing.fixtures import FutureEngineMixin
 from alembic.testing.fixtures import TestBase
-from alembic.util import compat
 
 
-class PatchEnvironment(object):
+class PatchEnvironment:
     branched_connection = False
 
     @contextmanager
@@ -117,14 +115,11 @@ class PatchEnvironment(object):
 
 
 @testing.combinations(
-    (False, True, False),
-    (True, False, False),
-    (True, True, False),
-    (False, True, True),
-    (True, False, True),
-    (True, True, True),
-    argnames="transactional_ddl,transaction_per_migration,branched_connection",
-    id_="rrr",
+    (False, True),
+    (True, False),
+    (True, True),
+    argnames="transactional_ddl,transaction_per_migration",
+    id_="rr",
 )
 class ApplyVersionsFunctionalTest(PatchEnvironment, TestBase):
     __only_on__ = "sqlite"
@@ -136,7 +131,9 @@ class ApplyVersionsFunctionalTest(PatchEnvironment, TestBase):
     branched_connection = False
 
     def setUp(self):
-        self.bind = _sqlite_file_db(future=self.future)
+        self.bind = _sqlite_file_db(
+            future=self.future, poolclass=pool.NullPool
+        )
         self.env = staging_env(sourceless=self.sourceless)
         self.cfg = _sqlite_testing_config(
             sourceless=self.sourceless, future=self.future
@@ -237,44 +234,56 @@ class ApplyVersionsFunctionalTest(PatchEnvironment, TestBase):
     def _test_002_upgrade(self):
         command.upgrade(self.cfg, self.c)
         db = self.bind
-        assert db.dialect.has_table(db.connect(), "foo")
-        assert db.dialect.has_table(db.connect(), "bar")
-        assert db.dialect.has_table(db.connect(), "bat")
+
+        with db.connect() as conn:
+            assert db.dialect.has_table(conn, "foo")
+            assert db.dialect.has_table(conn, "bar")
+            assert db.dialect.has_table(conn, "bat")
 
     def _test_003_downgrade(self):
         command.downgrade(self.cfg, self.a)
         db = self.bind
-        assert db.dialect.has_table(db.connect(), "foo")
-        assert not db.dialect.has_table(db.connect(), "bar")
-        assert not db.dialect.has_table(db.connect(), "bat")
+        with db.connect() as conn:
+            assert db.dialect.has_table(conn, "foo")
+            assert not db.dialect.has_table(conn, "bar")
+            assert not db.dialect.has_table(conn, "bat")
 
     def _test_004_downgrade(self):
         command.downgrade(self.cfg, "base")
         db = self.bind
-        assert not db.dialect.has_table(db.connect(), "foo")
-        assert not db.dialect.has_table(db.connect(), "bar")
-        assert not db.dialect.has_table(db.connect(), "bat")
+        with db.connect() as conn:
+            assert not db.dialect.has_table(conn, "foo")
+            assert not db.dialect.has_table(conn, "bar")
+            assert not db.dialect.has_table(conn, "bat")
 
     def _test_005_upgrade(self):
         command.upgrade(self.cfg, self.b)
         db = self.bind
-        assert db.dialect.has_table(db.connect(), "foo")
-        assert db.dialect.has_table(db.connect(), "bar")
-        assert not db.dialect.has_table(db.connect(), "bat")
+        with db.connect() as conn:
+            assert db.dialect.has_table(conn, "foo")
+            assert db.dialect.has_table(conn, "bar")
+            assert not db.dialect.has_table(conn, "bat")
 
     def _test_006_upgrade_again(self):
         command.upgrade(self.cfg, self.b)
         db = self.bind
-        assert db.dialect.has_table(db.connect(), "foo")
-        assert db.dialect.has_table(db.connect(), "bar")
-        assert not db.dialect.has_table(db.connect(), "bat")
+        with db.connect() as conn:
+            assert db.dialect.has_table(conn, "foo")
+            assert db.dialect.has_table(conn, "bar")
+            assert not db.dialect.has_table(conn, "bat")
 
     def _test_007_stamp_upgrade(self):
         command.stamp(self.cfg, self.c)
         db = self.bind
-        assert db.dialect.has_table(db.connect(), "foo")
-        assert db.dialect.has_table(db.connect(), "bar")
-        assert not db.dialect.has_table(db.connect(), "bat")
+        with db.connect() as conn:
+            assert db.dialect.has_table(conn, "foo")
+            assert db.dialect.has_table(conn, "bar")
+            assert not db.dialect.has_table(conn, "bat")
+
+
+class LegacyApplyVersionsFunctionalTest(ApplyVersionsFunctionalTest):
+    __requires__ = ("sqlalchemy_1x",)
+    branched_connection = True
 
 
 # class level combinations can't do the skips for SQLAlchemy 1.3
@@ -301,8 +310,7 @@ class SimpleSourcelessApplyVersionsTest(ApplyVersionsFunctionalTest):
     id_="r",
 )
 class NewFangledSourcelessApplyVersionsTest(ApplyVersionsFunctionalTest):
-
-    __requires__ = ("pep3147",)
+    pass
 
 
 class CallbackEnvironmentTest(ApplyVersionsFunctionalTest):
@@ -359,7 +367,7 @@ class CallbackEnvironmentTest(ApplyVersionsFunctionalTest):
         alembic.mock_event_listener = None
         self._env_file_fixture()
         with mock.patch("alembic.mock_event_listener", mock.Mock()) as mymock:
-            super(CallbackEnvironmentTest, self).test_steps()
+            super().test_steps()
         calls = mymock.call_args_list
         assert calls
         for call in calls:
@@ -373,7 +381,7 @@ class CallbackEnvironmentTest(ApplyVersionsFunctionalTest):
             assert isinstance(step.is_upgrade, bool)
             assert isinstance(step.is_stamp, bool)
             assert isinstance(step.is_migration, bool)
-            assert isinstance(step.up_revision_id, compat.string_types)
+            assert isinstance(step.up_revision_id, str)
             assert isinstance(step.up_revision, Script)
 
             for revtype in "up", "down", "source", "destination":
@@ -383,12 +391,12 @@ class CallbackEnvironmentTest(ApplyVersionsFunctionalTest):
                     assert isinstance(rev, Script)
                 revids = getattr(step, "%s_revision_ids" % revtype)
                 for revid in revids:
-                    assert isinstance(revid, compat.string_types)
+                    assert isinstance(revid, str)
 
             heads = kw["heads"]
             assert hasattr(heads, "__iter__")
             for h in heads:
-                assert h is None or isinstance(h, compat.string_types)
+                assert h is None or isinstance(h, str)
 
 
 class OfflineTransactionalDDLTest(TestBase):
@@ -533,7 +541,7 @@ def downgrade():
                 if self.is_sqlalchemy_future:
                     with testing.expect_raises_message(
                         sa.exc.InvalidRequestError,
-                        r"a transaction is already begun for this connection",
+                        r".*already",
                     ):
                         command.upgrade(self.cfg, c)
                 else:
@@ -555,7 +563,7 @@ def downgrade():
                 if self.is_sqlalchemy_future:
                     with testing.expect_raises_message(
                         sa.exc.InvalidRequestError,
-                        r"a transaction is already begun for this connection",
+                        r".*already",
                     ):
                         command.upgrade(self.cfg, c)
                 else:
@@ -577,7 +585,7 @@ def downgrade():
                 if self.is_sqlalchemy_future:
                     with testing.expect_raises_message(
                         sa.exc.InvalidRequestError,
-                        r"a transaction is already begun for this connection",
+                        r".*already",
                     ):
                         command.upgrade(self.cfg, c)
                 else:
@@ -622,6 +630,7 @@ run_migrations_online()
 
 
 class BranchedOnlineTransactionalDDLTest(OnlineTransactionalDDLTest):
+    __requires__ = ("sqlalchemy_1x",)
     branched_connection = True
 
 
@@ -644,8 +653,7 @@ class EncodingTest(TestBase):
             script,
             self.a,
             (
-                compat.u(
-                    """# coding: utf-8
+                """# coding: utf-8
 from __future__ import unicode_literals
 revision = '%s'
 down_revision = None
@@ -659,7 +667,6 @@ def downgrade():
     op.execute("drôle de petite voix m’a réveillé")
 
 """
-                )
                 % self.a
             ),
             encoding="utf-8",
@@ -673,7 +680,7 @@ def downgrade():
             bytes_io=True, output_encoding="utf-8"
         ) as buf:
             command.upgrade(self.cfg, self.a, sql=True)
-        assert compat.u("« S’il vous plaît…").encode("utf-8") in buf.getvalue()
+        assert "« S’il vous plaît…".encode() in buf.getvalue()
 
 
 class VersionNameTemplateTest(TestBase):
@@ -787,7 +794,7 @@ class IgnoreFilesTest(TestBase):
     sourceless = False
 
     def setUp(self):
-        self.bind = _sqlite_file_db()
+        self.bind = _sqlite_file_db(poolclass=pool.NullPool)
         self.env = staging_env(sourceless=self.sourceless)
         self.cfg = _sqlite_testing_config(sourceless=self.sourceless)
 
@@ -847,14 +854,10 @@ class SimpleSourcelessIgnoreFilesTest(IgnoreFilesTest):
 class NewFangledEnvOnlySourcelessIgnoreFilesTest(IgnoreFilesTest):
     sourceless = "pep3147_envonly"
 
-    __requires__ = ("pep3147",)
-
 
 class NewFangledEverythingSourcelessIgnoreFilesTest(IgnoreFilesTest):
     sourceless = "pep3147_everything"
 
-    __requires__ = ("pep3147",)
-
 
 class SourcelessNeedsFlagTest(TestBase):
     def setUp(self):
